@@ -3,7 +3,7 @@
 /*                  This file is part of the program and library             */
 /*         SCIP --- Solving Constraint Integer Programs                      */
 /*                                                                           */
-/*  Copyright (c) 2002-2025 Zuse Institute Berlin (ZIB)                      */
+/*  Copyright (c) 2002-2026 Zuse Institute Berlin (ZIB)                      */
 /*                                                                           */
 /*  Licensed under the Apache License, Version 2.0 (the "License");          */
 /*  you may not use this file except in compliance with the License.         */
@@ -316,16 +316,6 @@ SCIP_Real SCIPgetReoptSimilarity(
    int                   run2                /**< number of run */
    );
 
-/** check the changes of the variable coefficient in the objective function */
-SCIP_EXPORT
-void SCIPgetVarCoefChg(
-   SCIP*                 scip,               /**< SCIP data structure */
-   int                   varidx,             /**< index of variable */
-   SCIP_Bool*            negated,            /**< coefficient changed the sign */
-   SCIP_Bool*            entering,           /**< coefficient gets non-zero coefficient */
-   SCIP_Bool*            leaving             /**< coefficient gets zero coefficient */
-   );
-
 /** @} */
 
 #ifdef __cplusplus
